@@ -4,7 +4,7 @@ use Moose;
 use IO::Handle;
 use Text::CSV;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 BEGIN {
   # One day, MooseX::Getopt will allow us to pass pass_through to Getopt::Long.
@@ -16,49 +16,62 @@ BEGIN {
 
 with 'MooseX::Getopt';
 
+# Create "hasrw" and "hasro" sugar for less cumbersome attribute declarations.
 # Why isn't this in Moose?
-sub hasrw {
-  my($attr, @args) = @_;
-  has $attr => (
-    is => 'rw',
-    @args,
-  );
+BEGIN {
+  my $mk_has = sub {
+    my($access) = @_;
+    return sub {
+      my($attr, @args) = @_;
+      has $attr => (
+        is => $access,
+        metaclass => 'Getopt',  # For cmd_aliases
+        @args,
+      );
+    };
+  };
+  no strict 'refs';
+  *hasrw = $mk_has->('rw');
+  *hasro = $mk_has->('ro');
 }
 
-hasrw input => (isa => 'Str');
-
-hasrw output => (isa => 'Str');
+# Input and output filenames. Significant when we want to DWIM with TSV files.
+hasro input  => (isa => 'Str', cmd_aliases => 'i');
+hasro output => (isa => 'Str', cmd_aliases => 'o');
 
 # isa => 'FileHandle' (or IO::String...)
 hasrw _input_fh => ();
-
-# isa => 'FileHandle' (or IO::String...)
 hasrw _output_fh => ();
 
+# TODO: command line aliases?
+hasro from_tsv =>
+    (isa => 'Bool', cmd_aliases => 'from-tsv', predicate => 'has_from_tsv');
+hasro to_tsv   =>
+    (isa => 'Bool', cmd_aliases => 'to-tsv',   predicate => 'has_to_tsv');
+
 hasrw _init => (isa => 'Bool');
 
-hasrw columns => (isa => 'ArrayRef[Int]');
+# Normalized column indexes.
+hasrw columns => (isa => 'ArrayRef[Int]', cmd_aliases => 'c');
 
-# The input CSV processor.
-hasrw _input_csv => ();
-
-# The output CSV processor.
+# The input and output CSV processors.
+hasrw _input_csv  => ();
 hasrw _output_csv => ();
 
 # Text::CSV options, straight from the manpage.
 # We override Text::CSV's default here... because it's WRONG.
 our %TextCSVOptions = (
-    # name              => [type, default]
-    quote_char          => ['Str', '"'],
-    escape_char         => ['Str', '"'],
-    sep_char            => ['Str', ','],
-    eol                 => ['Any', undef],
+    # name              => [type, default, alias, @extra_opts]
+    quote_char          => ['Str', '"',   'q'],
+    escape_char         => ['Str', '"',   'e'],
+    sep_char            => ['Str', ',',   's', is => 'rw'],
+    eol                 => ['Any', ''],
     always_quote        => ['Int', 0],
-    binary              => ['Int', 1],
-    keep_meta_info      => ['Int', 0],
+    binary              => ['Int', 1,     'b'],
+    keep_meta_info      => ['Int', 0,     'k'],
     allow_loose_quotes  => ['Int', 0],
     allow_loose_escapes => ['Int', 0],
-    allow_whitespace    => ['Int', 0],
+    allow_whitespace    => ['Int', 0,     'w'],
     verbatim            => ['Int', 0],
 );
 
@@ -66,19 +79,34 @@ our %TextCSVOptions = (
 # But you can override it just for output by saying --output_foo instead
 # of --foo.   (Thanks, gphat and t0m.)
 while (my($attr, $opts) = each %TextCSVOptions) {
-  my($type, $default) = @$opts;
-  hasrw $attr => (isa => $type, default => $default);
-  hasrw "output_$attr" => (
+  my($type, $default, $short, @extra_opts) = @$opts;
+  hasro $attr => (
+    isa => $type,
+    default => $default,
+    __aliases($attr, $short),
+    @extra_opts
+  );
+  hasro "output_$attr" => (
     isa => $type,
     lazy => 1,
-    default => sub { $_[0]->$attr }
+    default => sub { $_[0]->$attr },
+    __output_aliases($attr),
+    @extra_opts,
   );
 }
 
-# TODO: command line aliases?
-hasrw from_tsv => (isa => 'Bool', predicate => 'has_from_tsv');
-hasrw to_tsv => (isa => 'Bool', predicate => 'has_to_tsv');
+sub __aliases {
+  my($attr, $short) = @_;
+  my @aliases;
+  (my $dashes = $attr) =~ s/_/-/g;
+  push @aliases, $dashes if $attr ne $dashes;
+  push @aliases, $short if $short;
+  return @aliases ? (cmd_aliases => \@aliases) : ();
+}
 
+sub __output_aliases {
+  return __aliases("output_" . shift);
+}
 
 sub __normalize_column {
   my($in) = @_;
@@ -162,6 +190,7 @@ sub run {
 }
 
 1;
+
 __END__
 =head1 NAME
 
